@@ -5,7 +5,7 @@
  *	MacOS X.
  *
  * Copyright 2001, Apple Computer, Inc.
- * Copyright (c) 2003-2009 Daniel A. Steffen <das@users.sourceforge.net>
+ * Copyright (c) 2003-2007 Daniel A. Steffen <das@users.sourceforge.net>
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -47,6 +47,8 @@
  *	authors grant the U.S. Government and others acting in its behalf
  *	permission to use and distribute the software in accordance with the
  *	terms specified in this license.
+ *
+ * RCS: @(#) $Id: tclMacOSXBundle.c,v 1.11.4.1 2008/12/07 16:39:32 das Exp $
  */
 
 #include "tclPort.h"
@@ -54,11 +56,6 @@
 #ifdef HAVE_COREFOUNDATION
 #include <CoreFoundation/CoreFoundation.h>
 #include <mach-o/dyld.h>
-
-#if MAC_OS_X_VERSION_MIN_REQUIRED < 1050
-MODULE_SCOPE long tclMacOSXDarwinRelease;
-#endif
-
 #endif /* HAVE_COREFOUNDATION */
 
 /*
@@ -148,16 +145,17 @@ Tcl_MacOSXOpenVersionedBundleResources(
 		    bundleVersion, kCFStringEncodingUTF8);
 
 	    if (bundleVersionRef) {
-		CFComparisonResult versionComparison = kCFCompareLessThan;
 		CFStringRef bundleTailRef = CFURLCopyLastPathComponent(
 			bundleURL);
 
 		if (bundleTailRef) {
-		    versionComparison = CFStringCompare(bundleTailRef,
-			    bundleVersionRef, 0);
+		    if (CFStringCompare(bundleTailRef, bundleVersionRef, 0) ==
+			    kCFCompareEqualTo) {
+			versionedBundleRef = (CFBundleRef) CFRetain(bundleRef);
+		    }
 		    CFRelease(bundleTailRef);
 		}
-		if (versionComparison != kCFCompareEqualTo) {
+		if (!versionedBundleRef) {
 		    CFURLRef versURL = CFURLCreateCopyAppendingPathComponent(
 			    NULL, bundleURL, CFSTR("Versions"), TRUE);
 
@@ -165,13 +163,9 @@ Tcl_MacOSXOpenVersionedBundleResources(
 			CFURLRef versionedBundleURL =
 				CFURLCreateCopyAppendingPathComponent(
 				NULL, versURL, bundleVersionRef, TRUE);
-
 			if (versionedBundleURL) {
 			    versionedBundleRef = CFBundleCreate(NULL,
 				    versionedBundleURL);
-			    if (versionedBundleRef) {
-				bundleRef = versionedBundleRef;
-			    }
 			    CFRelease(versionedBundleURL);
 			}
 			CFRelease(versURL);
@@ -181,6 +175,9 @@ Tcl_MacOSXOpenVersionedBundleResources(
 	    }
 	    CFRelease(bundleURL);
 	}
+	if (versionedBundleRef) {
+	    bundleRef = versionedBundleRef;
+	}
     }
 
     if (bundleRef) {
@@ -228,13 +225,7 @@ Tcl_MacOSXOpenVersionedBundleResources(
 	    CFRelease(libURL);
 	}
 	if (versionedBundleRef) {
-#if MAC_OS_X_VERSION_MIN_REQUIRED < 1050
-	    /* Workaround CFBundle bug in Tiger and earlier. [Bug 2569449] */
-	    if (tclMacOSXDarwinRelease >= 9)
-#endif
-	    {
-		CFRelease(versionedBundleRef);
-	    }
+	    CFRelease(versionedBundleRef);
 	}
     }
 
@@ -247,3 +238,11 @@ Tcl_MacOSXOpenVersionedBundleResources(
     return TCL_ERROR;
 #endif /* HAVE_COREFOUNDATION */
 }
+
+/*
+ * Local Variables:
+ * mode: c
+ * c-basic-offset: 4
+ * fill-column: 78
+ * End:
+ */
